#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include "outstreams.hpp"
#include "coordinates.hpp"
#include "SC2Map.hpp"


// next columsn bases
// % influence



void SC2Map::writeToCSV( const char* strOutCSV )
{
  char strNameCopy[FILENAME_LENGTH];
  strcpy( strNameCopy, this->strMapFilename );

  char* strNameAndCSV = strcat( strNameCopy, strOutCSV );

  FILE* fileCSV = fopen( strNameAndCSV, "w" );

  if( fileCSV == NULL )
  {
    printError( "Could not open %s for writing.\n", strOutCSV );
    return;
  }

  // write out column headers
  fprintf( fileCSV, "Start Location," );

  for( list<StartLoc*>::const_iterator itr = startLocs.begin();
       itr != startLocs.end();
       ++itr )
  {
    StartLoc* sl = *itr;

    fprintf( fileCSV, "Ground distance to %s WITH ROCKS,",                   sl->name );
    fprintf( fileCSV, "Cliff-walk distance to %s WITH ROCKS,",               sl->name );
    fprintf( fileCSV, "Air distance to %s,",                                 sl->name );
    fprintf( fileCSV, "Ground-CWalk Disparity %% to %s WITH ROCKS,",         sl->name );
    fprintf( fileCSV, "Ground-Air Disparity %% to %s WITH ROCKS,",           sl->name );
    fprintf( fileCSV, "Ground distance to %s NO ROCKS,",                     sl->name );
    fprintf( fileCSV, "Cliff-walk distance to %s NO ROCKS,",                 sl->name );
    fprintf( fileCSV, "%% Decrease Ground distance Remove Rocks to %s,",     sl->name );
    fprintf( fileCSV, "%% Decrease Cliff-walk distance Remove Rocks to %s,", sl->name );
  }

  fprintf( fileCSV, "Positional Balance %%\n" );

  // write a line for each start location
  for( list<StartLoc*>::const_iterator itr1 = startLocs.begin();
       itr1 != startLocs.end();
       ++itr1 )
  {
    StartLoc* sl1 = *itr1;

    fprintf( fileCSV, "%s,", sl1->name );

    for( list<StartLoc*>::const_iterator itr2 = startLocs.begin();
         itr2 != startLocs.end();
         ++itr2 )
    {
      StartLoc* sl2 = *itr2;

      if( sl1 == sl2 )
      {
        // don't compare start loc to itself!
        fprintf( fileCSV, "-,-,-,-,-,-,-,-,-," );
        continue;
      }


      float dGroundWR = getShortestPathDistance( &(sl1->loc), &(sl2->loc), PATH_GROUND_WITHROCKS );
      float dCWalkWR  = getShortestPathDistance( &(sl1->loc), &(sl2->loc), PATH_CWALK_WITHROCKS  );
      float dAir      = getShortestAirDistance ( &(sl1->loc), &(sl2->loc) );

      float Ground2CWalkDisparity = 100.0f * dCWalkWR / dGroundWR;
      float Ground2AirDisparity   = 100.0f * dAir     / dGroundWR;


      float dGroundNR = getShortestPathDistance( &(sl1->loc), &(sl2->loc), PATH_GROUND_NOROCKS );
      float dCWalkNR  = getShortestPathDistance( &(sl1->loc), &(sl2->loc), PATH_CWALK_NOROCKS  );

      float GroundDecreaseRemRocks = 100.0f * dGroundNR / dGroundWR;
      float CWalkDecreaseRemRocks  = 100.0f * dCWalkNR  / dCWalkWR;


      fprintf( fileCSV, "%f,", dGroundWR              );
      fprintf( fileCSV, "%f,", dCWalkWR               );
      fprintf( fileCSV, "%f,", dAir                   );
      fprintf( fileCSV, "%f,", Ground2CWalkDisparity  );
      fprintf( fileCSV, "%f,", Ground2AirDisparity    );
      fprintf( fileCSV, "%f,", dGroundNR              );
      fprintf( fileCSV, "%f,", dCWalkNR               );
      fprintf( fileCSV, "%f,", GroundDecreaseRemRocks );
      fprintf( fileCSV, "%f,", CWalkDecreaseRemRocks  );

    }

    fprintf( fileCSV, "%f\n", 95.0f );
  }

  fclose( fileCSV );
}
